#include "header/makemap.h"

using namespace std;

/* --------------------------------------------------------------
 * Function: 	MakeMap::MakeMap and MakeMap::~MakeMap()
 * Purpose: 	Constructor and destructor of MakeMap object
 * Arguments:	none
 * Return:		none
 * --------------------------------------------------------------*/
MakeMap::MakeMap()
{
	// defines the initial sonar state;
	sensorsState = false;
	sonar = new SenSonar();
	ir = new SenIr();
	cinematic = new Cinematic();
}

MakeMap::~MakeMap()
{
	delete sonar;
	delete cinematic;
	delete ir;
}


/* --------------------------------------------------------------
 * Function: 	MakeMap::newMap
 * Purpose: 	set the grid values of local map as unknow cell
 * Arguments:	none
 * Return:		int - status
 * --------------------------------------------------------------*/
int MakeMap::newMap()
{
	if (memset(mapa, CELL_UNKNO, sizeof(mapa)) == 0)
		return -1;
	else
		return 0;


}

/* --------------------------------------------------------------
 * Function: 	MakeMap::openMap
 * Purpose: 	fill the grid values of local map with file data
 * Arguments:	filename - the file name of map
 * Return:		int - status
 * --------------------------------------------------------------*/
int MakeMap::openMap(char* filename)
{

   ifstream infile(filename, ios::in);

   assert(infile); // make sure we could open the file

   // now read data.
   int inBit;
   for (int y = 0; y < MAP_HEIGHT; y++)
      for (int x = 0; x < MAP_WIDTH; x++) {
         infile >> inBit;
         assert(inBit >= -1 && inBit <= 11); // make sure data is valid
         mapa[(y*MAP_HEIGHT) + x] = inBit;
      }
	return 0;
}

/* --------------------------------------------------------------
 * Function: 	MakeMap::saveMap
 * Purpose: 	fill a file with data from local map
 * Arguments:	filename - the file name where the data will be save
 * Return:		int - status
 * --------------------------------------------------------------*/
int MakeMap::saveMap(char* filename)
{
   FILE* outfile = fopen(filename, "wt");

   assert(outfile); // make sure we could open the file

   // now write data.
   int inBit;
   for (int y = 0; y < MAP_HEIGHT; y++)
   {
      for (int x = 0; x < MAP_WIDTH; x++)
         fprintf(outfile, "%d ", mapa[(y*MAP_HEIGHT) + x]);
      fprintf(outfile, "\n");
   }
	return 0;

}

/* --------------------------------------------------------------
 * Function: 	MakeMap::getMap
 * Purpose: 	get local Map
 * Arguments:	none
 * Return:		int * - a pointer to local map
 * --------------------------------------------------------------*/
int * MakeMap::getMap()
{
	return mapa;
}



/* --------------------------------------------------------------
 * Function: 	MakeMap::paintNomad
 * Purpose: 	fill the cells in map to represent the robot
 * Arguments:	posx - the x coordinate of actual robot position
				posy - the y coordinate of actual robot position
 * Return:		none
 * --------------------------------------------------------------*/
void MakeMap::paintNomad(int posx, int posy)
{
	posx = int(posx/100);
	posy = int(posy/100);

	for (int i=posx-3; i< posx+3; i++)
	{
		mapa[(posy*MAP_HEIGHT) + i]     = CELL_ROBOT;
		mapa[((posy+1)*MAP_HEIGHT) + i] = CELL_ROBOT;
	}

	for (int j=posy-2; j<= posy+3;j++)
	{
		mapa[j*MAP_HEIGHT + posx]     = CELL_ROBOT;
		mapa[j*MAP_HEIGHT + (posx-1)] = CELL_ROBOT;
	}

	mapa[((posy-1)*MAP_HEIGHT) + (posx-2)] = CELL_ROBOT;
	mapa[((posy-1)*MAP_HEIGHT) + (posx+1)] = CELL_ROBOT;
	mapa[((posy+2)*MAP_HEIGHT) + (posx+1)] = CELL_ROBOT;
	mapa[((posy+2)*MAP_HEIGHT) + (posx-2)] = CELL_ROBOT;
}


/* --------------------------------------------------------------
 * Function: 	MakeMap::paintSensorData
 * Purpose: 	fill the map with sensor data
 * Arguments:	sensor        - the index of sensor read
				distance      - the distance read from sensor
				robotPosition - the coordinates of actual robot position
 * Return:		none
 * --------------------------------------------------------------*/
void MakeMap::paintSensorData(int sensor, int distance, Coord robotPosition)
{
	// validate data
	if ( (sensor > 24) || (sensor < 0) ) return;

	long posx = robotPosition.x;
	long posy = robotPosition.y;
	long rot  = robotPosition.rot;


	// Fill Limits
	float dist = distance;
	float limiteLimpo;
	float limiteTolerancia;

	// Sensor angles
	float ang  = sensor*(ARC*M_PI/180) + float(rot)/1000;
	float angI = ang +  (ARC*M_PI/180)/2;
	float angF = ang -  (ARC*M_PI/180)/2;

	int x;
	int y;



	if (distance >= READRADIUS)
	{
		distance = READRADIUS;
		limiteLimpo = 1;

		for (float i=angI; i > angF; i-=(M_PI/180)) // grau por grau
		{
			// fill the clear area
			for (float j=0; j<limiteLimpo; j+= 0.03)
			{
				x = (int)(((distance+300)*cos(i)*j) + posx)/100;
				y = (int)(((distance+300)*sin(i)*j) + posy)/100;
				// correct x,y if necessary
				x = (x < 0)?0:x;
				y = (y < 0)?0:y;
				mapa[(y*MAP_HEIGHT) + x] = CELL_CLEAR;
			}
		}
	}
	else
	{

		limiteLimpo = dist/(dist+300);
	 	limiteTolerancia = 1;

		for (float i=angI; i > angF; i-=(M_PI/180)) // grau por grau
		{
			// fill the clear area

			for (float j=0; j<limiteLimpo; j+= 0.03)
			{
				x = (int)(((distance+300)*cos(i)*j) + posx)/100;
				y = (int)(((distance+300)*sin(i)*j) + posy)/100;
				// correct x,y if necessary
				x = (x < 0)?0:x;
				y = (y < 0)?0:y;
				mapa[(y*MAP_HEIGHT) + x] = CELL_CLEAR;
			}
			// fill the tolerance area


			for (float j=limiteLimpo; j<limiteTolerancia; j+= 0.03)
			{
				x = (int)(((distance+300)*cos(i)*j) + posx)/100;
				y = (int)(((distance+300)*sin(i)*j) + posy)/100;
				// correct x,y if necessary
				x = (x < 0)?0:x;
				y = (y < 0)?0:y;
				mapa[(y*MAP_HEIGHT) + x] = CELL_TOLER;
			}



			// fill the obstacle area
			x = (int)(((distance+300)*cos(i)) + posx)/100;
			y = (int)(((distance+300)*sin(i)) + posy)/100;


			// correct x,y if necessary
			x = (x < 0)?0:x;
			y = (y < 0)?0:y;


			mapa[(y*MAP_HEIGHT) + x] = CELL_BUSY;

		}

	}

	// Paint the robot
	//paintNomad(posx, posy);


}


/* --------------------------------------------------------------
 * Function: 	MakeMap::refreshMap
 * Purpose: 	get the sensors data and refresh the actual map
 * Arguments:	none
 * Return:		none
 * --------------------------------------------------------------*/
void MakeMap::refreshMap(int posx, int posy)
{
        ENTER_CRITICALSECTION;
	state = N_GetRobotState(1);
	if (sensorsState == false)
	{
		// Sonars
		sonar = new SenSonar();
		int fo[8] = {0,1,2,3,4,5,6,7};
		for (int k = 0; k < 6; k++)
		if ( sonar->Start(state,k, fo, 0 ) != N_NO_ERROR )
			printf("Error in Sonar start. Sonar - %d\n", k );

		// Infrared
	   	if (ir->Start(state) != N_NO_ERROR )
			printf("Error in InfraRed start\n");

		sensorsState = true;

	}

	long distance;
	Coord robotPosition = cinematic->GetOdometry(state);
	LEAVE_CRITICALSECTION;
	robotPosition.x += posx;
	robotPosition.y += posy;


	for (int i=0; i< 24; i++)
	{
		// get the best values of sensors
		ENTER_CRITICALSECTION;
			distance = sonar->Read(state,i) <? ir->Read(state,i);
		LEAVE_CRITICALSECTION;
		paintSensorData(i, distance, robotPosition);
	}


}



/* --------------------------------------------------------------
 * Function: 	MakeMap::refreshMap2  No Critical Session
 * Purpose: 	get the sensors data and refresh the actual map
 * Arguments:	none
 * Return:		none
 * --------------------------------------------------------------*/
void MakeMap::refreshMap2(int posx, int posy)
{

	state = N_GetRobotState(1);

	long distance;
	Coord robotPosition = cinematic->GetOdometry(state);
	robotPosition.x += posx;
	robotPosition.y += posy;

	N_GetSonar(1);
	N_GetInfrared(1);
	state = N_GetRobotState(1);

	for (int i=0; i< 24; i++)
	{
		// get the best values of sensors
		distance = sonar->Read(state,i) <? ir->Read(state,i);
		paintSensorData(i, distance, robotPosition);
	}

}


void MakeMap::finishThread()
{

	ENTER_CRITICALSECTION;
		state = N_GetRobotState(1);
		sonar->StopAll(state);
		sensorsState = false;
	LEAVE_CRITICALSECTION;
}



