/******************************************************************************

SENSERVER.C - Robot's sensor server through the INETD main server
 Sergio Roberto Gonsalves Tourino - tourino@graco.unb.br - 10/05/2000

Before using this program, edit the 'inetd.conf' and the 'services' files:

INETD.CONF:
 senserver  stream  tcp     nowait  root /home/tourino/senserver senserver

SERVICES:
 senserver  8085/tcp

After these modifications, restart the inetd server:
 kill -1 <PID of inetd - see with ps>

*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "Nclient.h"

#define HOST	"lower"
#define PORT	7073

#define NONE		0
#define FRONT		1
#define BACK		2
#define LEFT		3
#define RIGHT		4
#define SPINP		5
#define SPINN		6

/* #define DEBUG */

int disconnect (void);
void get_battery (void);
void get_IR (void);
void get_sonar (void);
void get_odometry (void);
void get_speed (void);

/*-----------------3/29/00 5:17PM-------------------
 * This main routine parses commands from stdin, which
 * corresponds to a socket connection from a remote client
 * connected to this server. Its outputs are sent to the
 * stdout stream, back to the client machine. The local
 * tests are made with the stderr stream. This program is
 * to be used with the INETD main server.
 * --------------------------------------------------*/

int sensor_on = FALSE;		/* global variable to verify sensor's state */

/****************************************************************************/

void goodbye (void) {
	int result;

    	result = system ("rm /home/tourino/senserver.lock");
}

/****************************************************************************/

int main (void) {

    const char delimiters[] = "=, ";    /* delimiters for the commands */
    char command[] = "direction=front, distance=500, nothing important, direction=left, goodbye";
    char *token, parameters[10], commandline[100];
    char direction[10];                 /* direction for the movement */
    
    char sensors[3];			/* state of sensors */
    char get[10];

    int open_connection = TRUE;
    int result;
    int dir, distance = 500, speed=100, teta=524, omega=314;	/* movement configuration */

    FILE *lock;

    /* verify the lock file */

    lock = fopen ("/home/tourino/senserver.lock","r");

    if (lock == NULL) {                 /* the lock does not exist */
        lock = fopen ("/home/tourino/senserver.lock","w+");
        fclose (lock);
    }
    else {                              /* the lock exists */
        fclose (lock);
        fprintf (stderr,"Lock file found. Program aborted.\n");
        exit (0);                       /* closes the program */
    }

    atexit (goodbye);		/* defines the 'goodbye' function to be
					called at program termination */

    N_InitializeClient (HOST,PORT);	/* opening robot's connection */
    result = N_ConnectRobot (1);

	switch (result) {
		case N_NO_ERROR:
			fprintf (stderr,"Success!\n");
			break;
		case N_UNINITIALIZED:
			fprintf (stderr,"Not initialized!\n");
			exit(0);
			break;
		case N_ROBOT_NOT_FOUND:
			fprintf (stderr,"Robot not found!\n");
			exit(0);
			break;
		case N_CONNECTION_FAILED:
			fprintf (stderr,"Connection failed!\n");
			exit(0);
			break;
		case N_OUT_OF_MEMORY:
			fprintf (stderr,"Out of memory!\n");
			exit(0);
			break;
	}

    while (open_connection) {           /* loop until 'open_connection' is false */

        fscanf (stdin,"%s",command);    /* reads commands from stdin */

        token = strtok(command,delimiters);  /* strtok parses the commands from 'command' variable */

        while ((token != NULL) && open_connection) {

	    if (strcmp(token,"get") == 0) {
		strcpy (get,strtok(NULL,delimiters));
		if (strcmp (get,"battery") == 0) {
			get_battery ();
		}
		if (strcmp (get,"IR") == 0) {
			get_IR ();
		}
		if (strcmp (get,"sonar") == 0) {
			get_sonar ();
		}
		if (strcmp (get,"odometry") == 0) {
			get_odometry ();
		}
		if (strcmp (get,"speed") == 0) {
			get_speed ();
		}
	    }

            if (strcmp(token,"goodbye") == 0) {
                open_connection = FALSE;  /* terminates all of the loops */
            }

            token = strtok (NULL,delimiters);
        }
    }

    fprintf (stderr,"Goodbye. Connection closed.\n");

    result = disconnect ();

    return 0;
}

/****************************************************************************/

struct MySonar {
	struct N_Sonar	sonar[N_MAX_SONAR_COUNT];
};

struct MyBumper {
	struct N_Bumper bumper[N_MAX_BUMPER_COUNT];
};

struct Map {
	int Reading[24];
};

void ReadSonarSet (int ss_number, struct MySonar *sonarset);
void ShowSonarSet (struct MySonar sonarset);
void MinSonarSet (struct MySonar sonar1, struct MySonar sonar2, 
	struct MySonar *minsonar);
void MapSonarSet (struct MySonar sonar0, struct MySonar sonar1, 
		struct MySonar sonar2, struct Map *map);
void ShowMap (struct Map map);
void MakeSonarMap (struct Map *map);
void ShowSonarMap (struct Map map);
int MapAngle (int angle);
int GetSMapReading (struct Map map, int angle, int n_sensors);
void ShowEvidence (struct Map map);

/****************************************************************************/

struct MyIR {
	struct N_Infrared	IR [N_MAX_SONAR_COUNT];
};

/****************************************************************************/

void ReadIRSet (int ss_number, struct MyIR *IRset);
void ShowIRSet (struct MyIR IRset);
void MaxIRSet (struct MyIR IR1, struct MyIR IR2, 
	struct MyIR *minIR);
void MapIRSet (struct MyIR IR0, struct MyIR IR1, 
		struct MyIR IR2, struct Map *map);
void ShowIRMap (struct Map map);
void MakeIRMap (struct Map *map);
void MakeBumperMap (struct Map *map);
int GetIMapReading (struct Map map, int angle, int n_sensors);

/****************************************************************************/

#define X	N_XTRANSLATION
#define Y	N_YTRANSLATION
#define T	N_ROTATION

#define MIN_DIST	200		/* value in milimeters 	*/
#define MAX_INT		50		/* max value is 255	*/

#define TO_RADIAN	3.14159265358979/180

/****************************************************************************/

/* Routine that closes the connection with the XRDev */

int disconnect (void)
{
	int result;
	result = N_DisconnectRobot (1);
	
	switch (result) {
		case N_NO_ERROR:
			printf ("Success!\n");
			break;
		case N_CONNECTION_FAILED:
			printf ("Connection lost!\n");
			break;
		case N_ROBOT_NOT_FOUND:
			printf ("Robot not found!\n");
			break;
	}

	return (result);
}

/****************************************************************************/

/* Routine used to get the batteries' voltages */

void get_battery (void) {
	
	struct N_RobotState *state;
	struct N_Battery *battery;
	int i;

	state = N_GetRobotState (1);

	fprintf (stdout," B");
	for (i=0;i<N_MAX_BATTERY_COUNT;i++) {
		battery = &(state->BatterySet.Battery[i]);
		N_GetBattery (1);
		fprintf (stdout," %d",battery->Voltage/100);
	}
	fprintf (stdout," B ");

	fflush (stdout);
}

/****************************************************************************/

void get_IR (void) {
	
	struct Map mapI;
	int i;

	MakeIRMap (&mapI);

	fprintf (stdout," I");
	for (i=0;i<24;i++) 
		fprintf (stdout," %d",mapI.Reading[i]/10);

	fprintf (stdout," I ");
	fflush (stdout);
}

/****************************************************************************/

void get_sonar (void) {

	struct Map mapS;
	int i;

#ifdef DEBUG
	FILE *outfile;

	outfile = fopen ("/home/tourino/sensors.out","a");
#endif

	MakeSonarMap (&mapS);

	fprintf (stdout," S");
	for (i=0;i<24;i++) {
		fprintf (stdout," %d",mapS.Reading[i]/100);
#ifdef DEBUG
		fprintf (outfile," %d",mapS.Reading[i]);
#endif
	}

	fprintf (stdout," S ");
	fflush (stdout);

#ifdef DEBUG
	fprintf (outfile,"\n");
	fflush (outfile);
	fclose (outfile);
#endif

}

/****************************************************************************/

void get_odometry (void) {

	struct N_RobotState *state;
	struct N_Integrator *integrated;

	state = N_GetRobotState (1);
	integrated = &(state->Integrator);

	N_GetIntegratedConfiguration (1);
	fprintf (stdout," O %d %d %d %d O ",
		integrated->x, integrated->y, 
		integrated->Steering, integrated->Rotation);
	fflush (stdout);
}


/****************************************************************************/

void get_speed (void) {

	struct N_RobotState *state;
	struct N_Axis *axis;

	state = N_GetRobotState(1);
	
	axis = &(state->AxisSet.Axis[X]);
	N_GetAxes(1);
	fprintf (stdout," V");
	fprintf (stdout," %d ",axis->ActualVelocity);

	axis = &(state->AxisSet.Axis[Y]);
	N_GetAxes(1);
	fprintf (stdout,"%d ",axis->ActualVelocity);

	axis = &(state->AxisSet.Axis[T]);
	N_GetAxes(1);
	fprintf (stdout,"%d V ",axis->ActualVelocity);

	fflush(stdout);
}

/****************************************************************************/

/* Gets the lower value of the sonar sets */

void MinSonarSet (struct MySonar sonar1, struct MySonar sonar2, 
	struct MySonar *minsonar) {

int	i;

for (i=0;i<8;i++) {
	minsonar->sonar[i].Reading = sonar1.sonar[i].Reading <
		sonar2.sonar[i].Reading?sonar1.sonar[i].Reading:
		sonar2.sonar[i].Reading;
}

}

/****************************************************************************/

/* Reads sonar data and stores it in the MySonar structure */

void ReadSonarSet (int ss_number, struct MySonar *sonarset) {

struct N_RobotState	*state;	
struct N_Sonar		*sonar;
int			i;

state = N_GetRobotState (1);
sonar = state->SonarController.SonarSet[ss_number].Sonar;
N_GetSonar (1);

for (i=0;i<8;i++)
	sonarset->sonar[i].Reading=sonar[i].Reading;

}

/****************************************************************************/

/* Reads bumper data and stores it in the MyBumper structure */

void ReadBumperSet (int bs_number, struct MyBumper *bumperset) {

struct N_RobotState 	*state;
struct N_Bumper		*bumper;
int			i;

state = N_GetRobotState (1);
bumper = state->BumperController.BumperSet[bs_number].Bumper;
N_GetBumper (1);

for (i=0;i<8;i++)
	bumperset->bumper[i].Reading=bumper[i].Reading;

}

/****************************************************************************/

/* Prints out the sonar data to the stderr output */

void ShowSonarSet (struct MySonar sonarset) {

int	i;

for (i=0;i<8;i++) 
	fprintf (stderr,"%d ",sonarset.sonar[i].Reading);
fprintf (stderr,"\n");

}

/****************************************************************************/

/* Translates the sonar data into MySonar structure to the Map structure */

void MapSonarSet (struct MySonar sonar0, struct MySonar sonar1, 
		struct MySonar sonar2, struct Map *map) {

int	angle, i;

angle = 0;

for (i=6;i<8;i++) {
	map->Reading[angle] = sonar2.sonar[i].Reading;
	angle++;
}

for (i=0;i<8;i++) {
	map->Reading[angle] = sonar0.sonar[i].Reading;
	angle++;
}

for (i=0;i<8;i++) {
	map->Reading[angle] = sonar1.sonar[i].Reading;
	angle++;
}

for (i=0;i<6;i++) {
	map->Reading[angle] = sonar2.sonar[i].Reading;
	angle++;
}

}

/****************************************************************************/

/* Translates the bumper data into MyBumper structure to the Map structure
*/

void MapBumperSet (struct MyBumper bumper0, struct MyBumper bumper1, 
		struct MyBumper bumper2, struct Map *map) {

int	angle, i;

angle = 0;

for (i=6;i<8;i++) {
	map->Reading[angle] = bumper2.bumper[i].Reading;
	angle++;
}

for (i=0;i<8;i++) {
	map->Reading[angle] = bumper0.bumper[i].Reading;
	angle++;
}

for (i=0;i<8;i++) {
	map->Reading[angle] = bumper1.bumper[i].Reading;
	angle++;
}

for (i=0;i<6;i++) {
	map->Reading[angle] = bumper2.bumper[i].Reading;
	angle++;
}

}

/****************************************************************************/

/* Prints out the Map structure to the stderr output */

void ShowMap (struct Map map) {

int 	i;

for (i=0;i<24;i++)
	fprintf (stderr,"%.1f",map.Reading[i]>10000?10:map.Reading[i]/1000.0);

fprintf (stderr,"\n");
}

/****************************************************************************/

/* Returns the greater value from the MyBumper structures */

void MaxBumperSet (struct MyBumper bumper1, struct MyBumper bumper2, 
	struct MyBumper *minbumper) {

int	i;

for (i=0;i<8;i++) {
	minbumper->bumper[i].Reading = bumper1.bumper[i].Reading >
		bumper2.bumper[i].Reading?bumper1.bumper[i].Reading:bumper2.bumper[i].Reading;
}

}

/****************************************************************************/

/* Translate all the bumper sets into the Map structure */

void MakeBumperMap (struct Map *map) {

struct	MyBumper	bs,bi,bumper0,bumper1,bumper2;

ReadBumperSet (0,&bs);
ReadBumperSet (1,&bi);
MaxBumperSet (bs,bi,&bumper0);
ReadBumperSet (2,&bs);
ReadBumperSet (3,&bi);
MaxBumperSet (bs,bi,&bumper1);
ReadBumperSet (4,&bs);
ReadBumperSet (5,&bi);
MaxBumperSet (bs,bi,&bumper2);

MapBumperSet (bumper0,bumper1,bumper2,map);

}

/****************************************************************************/

/* Translate all the sonar sets into the Map structure */

void MakeSonarMap (struct Map *map) {

struct	MySonar		ss,si,sonar0,sonar1,sonar2;

ReadSonarSet (0,&ss);
ReadSonarSet (1,&si);
MinSonarSet (ss,si,&sonar0);
ReadSonarSet (2,&ss);
ReadSonarSet (3,&si);
MinSonarSet (ss,si,&sonar1);
ReadSonarSet (4,&ss);
ReadSonarSet (5,&si);
MinSonarSet (ss,si,&sonar2);

MapSonarSet (sonar0,sonar1,sonar2,map);

}

/****************************************************************************/

/* Displays a bar graph of the sonar data into the stderr output */

void ShowSonarMap (struct Map map) {

short	i,j;
char	s[80];

for (i=0;i<24;i++) {
	strcpy (s,"");
      	for (j=0;j < (map.Reading[i]>10000?10000:map.Reading[i])*0.006;j++)
        	strcat (s,"-");
      	fprintf (stderr,"%d\t%s\n",(map.Reading[i]>10000?10000:map.Reading[i]),s);
}

refresh();
usleep (100000);
clear ();

}

/****************************************************************************/

/* Maps a provided angle to the Map structure's index */

int MapAngle (int angle) {

int a;
	a = ceil (0.06969*angle-1.04545);
	a = a < 0 ? 24+a : a;
	return (a);
}

/****************************************************************************/

/* Gets the value of sonar data in a specified direction */

int GetSMapReading (struct Map map, int angle, int n_sensors) {

int	s, i, a, n, t;

n = n_sensors / 2;
a = MapAngle (angle);
s = 10000;				/* max value to sonar data */
for (i=0;i<2*n+1;i++) {
	t = a-n+i;
	t = t < 0 ? 24-t : t;
	t = t > 23 ?  t-24 : t;
	s = map.Reading[t]<s ? map.Reading[t] : s;
}
return (s);

}

/****************************************************************************/

/* Gets the value of IR data in a specified direction */

int GetIMapReading (struct Map map, int angle, int n_sensors) {

int	s, i, a, n, t;

n = n_sensors / 2;
a = MapAngle (angle);
s = 0;				/* min value for IR data */
for (i=0;i<2*n+1;i++) {
	t = a-n+i;
	t = t < 0 ? 24-t : t;
	t = t > 23 ?  t-24 : t;
	s = map.Reading[t]>s ? map.Reading[t] : s;
}
return (s);

}

/****************************************************************************/

/* Gets the value of Bumper data in a specified direction */

int GetBMapReading (struct Map map, int angle, int n_sensors) {

int	s, i, a, n, t;

n = n_sensors / 2;
a = MapAngle (angle);
s = N_BUMPER_NONE;			/* min value for bumper data */
for (i=0;i<2*n+1;i++) {
	t = a-n+i;
	t = t < 0 ? 24-t : t;
	t = t > 23 ?  t-24 : t;
	s = map.Reading[t]>s ? map.Reading[t] : s;
}
return (s);

}

/****************************************************************************/

/* Displays a gridmap containing the values of Map structure */

void ShowEvidence (struct Map map) {

int 	i,j,l,xm,ym;
int 	EGrid [20][20];
char 	s[80];

for (i=0; i < 20; i++) 
	for (j=0; j < 20; j++) 
		EGrid [i][j] = 0;

EGrid [10][10] = 2;

for (i=0; i < 360; i+= 15) {
	l = GetSMapReading (map, i, 1);
	xm = (3 + (l+300) * cos (i*TO_RADIAN) / 1000) * 3;
	ym = (3 + (l+300) * sin (i*TO_RADIAN) / 1000) * 3;
	if ((xm < 20 && xm >= 0) && (ym >= 0 && ym < 20))
		EGrid [xm][ym] = 1;
}

/* plotting from up to down... */

for (j=19; j >= 0; j--) {
	strcpy (s,"");
	for (i=0; i < 20; i++) {
		if (EGrid [i][j] == 1)
			strcat (s,"++");
		else
			if (EGrid [i][j] == 2)
				strcat (s,"OO");
			else
				strcat (s,"..");
	}
	fprintf (stderr,"%s\n",s);
}

usleep (200000);

}

/****************************************************************************/

/* Returns the greater value from the MyIR structures */

void MaxIRSet (struct MyIR IR1, struct MyIR IR2, 
	struct MyIR *minIR) {

int	i;

for (i=0;i<8;i++) {
	minIR->IR[i].Reading = IR1.IR[i].Reading >
		IR2.IR[i].Reading?IR1.IR[i].Reading:IR2.IR[i].Reading;
}

}

/****************************************************************************/

/* Translates the IR data into the MyIR structure */

void ReadIRSet (int ss_number, struct MyIR *IRset) {

struct N_RobotState	*state;	
struct N_Infrared	*IR;
int			i;

state = N_GetRobotState (1);
IR = state->InfraredController.InfraredSet[ss_number].Infrared;
N_GetInfrared (1);

for (i=0;i<8;i++)
	IRset->IR[i].Reading=IR[i].Reading;

}

/****************************************************************************/

/* Prints out the IR data from a MyIR structure */

void ShowIRSet (struct MyIR IRset) {

int	i;

for (i=0;i<8;i++) 
	fprintf (stderr,"%d ",IRset.IR[i].Reading);
fprintf (stderr,"\n");

}

/****************************************************************************/

/* Translates MyIR structures into the Map structure */

void MapIRSet (struct MyIR IR0, struct MyIR IR1, 
		struct MyIR IR2, struct Map *map) {

int	angle, i;

angle = 0;

for (i=6;i<8;i++) {
	map->Reading[angle] = IR2.IR[i].Reading;
	angle++;
}

for (i=0;i<8;i++) {
	map->Reading[angle] = IR0.IR[i].Reading;
	angle++;
}

for (i=0;i<8;i++) {
	map->Reading[angle] = IR1.IR[i].Reading;
	angle++;
}

for (i=0;i<6;i++) {
	map->Reading[angle] = IR2.IR[i].Reading;
	angle++;
}

}

/****************************************************************************/

/* Prints out the Map structure for the IR data */

void ShowIRMap (struct Map map) {

int 	i;

for (i=0;i<24;i++)
	fprintf (stderr,"%d ",map.Reading[i]);

fprintf (stderr,"\n");
}

/****************************************************************************/

/* Translates all the IR data into the Map structure */

void MakeIRMap (struct Map *map) {

struct	MyIR	ss,si,IR0,IR1,IR2;

ReadIRSet (0,&ss);
ReadIRSet (1,&si);
MaxIRSet (ss,si,&IR0);
ReadIRSet (2,&ss);
ReadIRSet (3,&si);
MaxIRSet (ss,si,&IR1);
ReadIRSet (4,&ss);
ReadIRSet (5,&si);
MaxIRSet (ss,si,&IR2);

MapIRSet (IR0,IR1,IR2,map);

}	

