/******************************************************************************

ROBSERVER.C - Robot's server through the INETD main server
 Sergio Roberto Gonsalves Tourino - tourino@graco.unb.br - 10/05/2000

Before using this program, edit the 'inetd.conf' and the 'services' files:

INETD.CONF:
 robserver  stream  tcp     nowait  root /home/tourino/robserver robserver

SERVICES:
 robserver  8086/tcp

After these modifications, restart the inetd server:
 kill -1 <PID do inetd - ver com ps>

*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <unistd.h>
#include <signal.h>
#include "Nclient.h"

#define HOST	"lower"
#define PORT	7073

#define NONE		0
#define FRONT		1
#define BACK		2
#define LEFT		3
#define RIGHT		4
#define SPINP		5
#define SPINN		6

void robot_panel (char *message);
void start_sensors (void);
void stop_sensors (void);
int disconnect (void);
void get_battery (void);
void get_IR (void);
void get_sonar (void);
void move_robot (int direction, int distance, int speed, int teta, int omega);

/*-----------------3/29/00 5:17PM-------------------
 * This main routine parses commands from stdin, which
 * corresponds to a socket connection from a remote client
 * connected to this server. Its outputs are sent to the
 * stdout stream, back to the client machine. The local
 * tests are made with the stderr stream. This program is
 * to be used with the INETD main server.
 * --------------------------------------------------*/

int sensor_on = FALSE;		/* global variable to verify sensor's state */
int stopnow = FALSE;		/* global variable to stop the robot */

/****************************************************************************/

void goodbye (void) {
	int result;

        result = system ("rm /home/tourino/robserver.lock");
}

/****************************************************************************/

void stoprobot () {
	
	stopnow = TRUE;
	printf ("Robot stopped!\n");
}

/****************************************************************************/

int main (void) {

    const char delimiters[] = "=, ";    /* delimiters for the commands */
    char command[] = "direction=front, distance=500, nothing important, direction=left, goodbye";
    char *token, parameters[10], commandline[100];
    char direction[10];                 /* direction for the movement */
    
    char sensors[3];			/* state of sensors */
    char get[10];

    int open_connection = TRUE;
    int result,pid;
    int dir, distance = 500, speed=100, teta=524, omega=314;	/* movement configuration */

    FILE *connection, *lock, *mypid;

    /* verify the lock file */

    lock = fopen ("/home/tourino/robserver.lock","r");

    if (lock == NULL) {                 /* the lock does not exist */
        lock = fopen ("/home/tourino/robserver.lock","w+");
        fclose (lock);
    }
    else {                              /* the lock exists */
        fclose (lock);
        fprintf (stderr,"Lock file found. Program aborted.\n");
        exit (0);                       /* closes the program */
    }

    atexit (goodbye);		/* defines the function 'goodbye' to be
					called at program termination */

    /* create the PID file */

    pid = getpid ();        /* getting the pid of program */

    /* saving pid */
    mypid = fopen ("/home/tourino/robserver.pid","w+");
    fprintf (mypid,"%d",pid);
    fclose (mypid);

    /* defines the listener for the stop signal from the stop server */

    signal (SIGUSR1,stoprobot);   

    connection = fopen ("/home/tourino/robserver.out","w+"); /* opening file */

    N_InitializeClient (HOST,PORT);	/* opening robot's connection */
    result = N_ConnectRobot (1);

	switch (result) {
		case N_NO_ERROR:
			fprintf (stderr,"Success!\n");
			break;
		case N_UNINITIALIZED:
			fprintf (stderr,"Not initialized!\n");
			exit(0);
			break;
		case N_ROBOT_NOT_FOUND:
			fprintf (stderr,"Robot not found!\n");
			exit(0);
			break;
		case N_CONNECTION_FAILED:
			fprintf (stderr,"Connection failed!\n");
			exit(0);
			break;
		case N_OUT_OF_MEMORY:
			fprintf (stderr,"Out of memory!\n");
			exit(0);
			break;
	}

    while (open_connection) {           /* loop until 'open_connection' is false */

        fscanf (stdin,"%s",command);    /* reads commands from stdin */
	fprintf (connection,"%s\n",command);	/* writes to file */
	fflush (connection);			/* it's better... */

        token = strtok(command,delimiters);  /* strtok parses the commands from 'command' variable */

        while ((token != NULL) && open_connection) {

            if (strcmp(token,"sensors") == 0) {
                strcpy (sensors,strtok(NULL,delimiters));
		if (strcmp (sensors,"on") == 0) {
			start_sensors ();
			sensor_on = TRUE;
		}
		if (strcmp (sensors,"off") == 0) {
			stop_sensors ();

/* since who is turning off the sensors will stop the session when I
remove the lock file sooner... */

		        result = system ("rm /home/tourino/robserver.lock");
			sensor_on = FALSE;
		}
            }

	    if (strcmp(token,"direction") == 0) {
		strcpy (direction,strtok(NULL,delimiters));
	    }

	    if (strcmp(token,"distance") == 0) {
		distance = atoi (strtok (NULL,delimiters));
	    }

	    if (strcmp(token,"speed") == 0) {
		speed = atoi (strtok (NULL,delimiters));
	    }

	    if (strcmp(token,"teta") == 0) {
		teta = atoi (strtok (NULL,delimiters));
	    }

	    if (strcmp(token,"omega") == 0) {
		omega = atoi (strtok (NULL,delimiters));
	    }

            if (strcmp(token,"start") == 0) {
                /* Here comes the commands for the control of the robot.
                 * This command MUST only work with all the variables (speed, acceleration, ...)
                 * adjusted accordingly. */
		if (strcmp(direction,"front") == 0)
			dir = FRONT;
		if (strcmp(direction,"back") == 0)
			dir = BACK;
		if (strcmp(direction,"left") == 0)
			dir = LEFT;
		if (strcmp(direction,"right") == 0)
			dir = RIGHT;
		if (strcmp(direction,"clockwise") == 0)
			dir = SPINN;
		if (strcmp(direction,"counterclockwise") == 0)
			dir = SPINP;

		if (sensor_on) {
			move_robot (dir,distance,speed,teta,omega);	/* moves the robot */
			strcpy (direction,"");		/* clears the variable */
			dir = NONE;
		}
		else {
			fprintf (stderr,"Error: sensors turned off.\n");
		}
            }

            if (strcmp(token,"goodbye") == 0) {
                open_connection = FALSE;  /* terminates all of the loops */
            }

            token = strtok (NULL,delimiters);
        }
    }

    fprintf (stderr,"Goodbye. Connection closed.\n");

    fclose (connection);
    stop_sensors();

    result = disconnect ();

    return 0;
}

/****************************************************************************/

struct MySonar {
	struct N_Sonar	sonar[N_MAX_SONAR_COUNT];
};

struct MyBumper {
	struct N_Bumper bumper[N_MAX_BUMPER_COUNT];
};

struct Map {
	int Reading[24];
};

void ReadSonarSet (int ss_number, struct MySonar *sonarset);
void ShowSonarSet (struct MySonar sonarset);
void MinSonarSet (struct MySonar sonar1, struct MySonar sonar2, 
	struct MySonar *minsonar);
void MapSonarSet (struct MySonar sonar0, struct MySonar sonar1, 
		struct MySonar sonar2, struct Map *map);
void ShowMap (struct Map map);
void MakeSonarMap (struct Map *map);
void ShowSonarMap (struct Map map);
int MapAngle (int angle);
int GetSMapReading (struct Map map, int angle, int n_sensors);
void ShowEvidence (struct Map map);

/****************************************************************************/

struct MyIR {
	struct N_Infrared	IR [N_MAX_SONAR_COUNT];
};

/****************************************************************************/

void ReadIRSet (int ss_number, struct MyIR *IRset);
void ShowIRSet (struct MyIR IRset);
void MaxIRSet (struct MyIR IR1, struct MyIR IR2, 
	struct MyIR *minIR);
void MapIRSet (struct MyIR IR0, struct MyIR IR1, 
		struct MyIR IR2, struct Map *map);
void ShowIRMap (struct Map map);
void MakeIRMap (struct Map *map);
void MakeBumperMap (struct Map *map);
int GetIMapReading (struct Map map, int angle, int n_sensors);

/****************************************************************************/

#define X	N_XTRANSLATION
#define Y	N_YTRANSLATION
#define T	N_ROTATION

#define MIN_DIST	300		/* value in milimeters 	*/
#define MAX_INT		30		/* max value is 255	*/

#define TO_RADIAN	3.14159265358979/180

/****************************************************************************/

/* Routine used to start the sonar sensors */

void start_sensors (void)
{
	struct N_RobotState *state;
	struct N_SonarSet *sonar_set;
	int i,j;

	state = N_GetRobotState(1);
	
	for (i=0;i<6;i++) {	
	
		sonar_set = &(state->SonarController.SonarSet[i]);
		N_GetSonarConfiguration(1);

		sonar_set->DataActive = TRUE;
		sonar_set->TimeStampActive = TRUE;

		for (j=0;j<=7;j++) 
			sonar_set->FiringOrder[j] = j;
		sonar_set->FiringOrder[8] = N_END_SONAR_FIRING_ORDER;
		
		N_SetSonarConfiguration(1);

		fprintf (stderr,"Sonar set %d set to\n",i);

		fprintf (stderr,"Firing order: ");
		j = 0;
		while (sonar_set->FiringOrder[j] !=
N_END_SONAR_FIRING_ORDER) {
			fprintf (stderr,"%d ",sonar_set->FiringOrder[j]);
			j++;
		}
	}
}

/****************************************************************************/

/* Routine used to turn off the sonar sensors */

void stop_sensors (void)
{
	struct N_RobotState *state;
	struct N_SonarSet *sonar_set;
	int i;

	state = N_GetRobotState(1);

	for (i=0;i<6;i++) {	
		sonar_set = &(state->SonarController.SonarSet[i]);
		N_GetSonarConfiguration(1);

		sonar_set->DataActive = TRUE;
		sonar_set->TimeStampActive = TRUE;
		sonar_set->FiringOrder[0] = N_END_SONAR_FIRING_ORDER;

		N_SetSonarConfiguration(1);
	}
}

/****************************************************************************/

/* Routine used to write to the robot's panel */

void robot_panel (char *message)
{
	char 	*to_panel;
	int 	result;

	strcpy (to_panel,"/usr/local/xrdev/bin/cpnl_log '");
	strcat (to_panel,message);
	strcat (to_panel,"'");

	result = system (to_panel);

	if (result == -1 || result == 127)
		fprintf (stderr,"Error while calling 'cpnl_log' program.\n");
}

/****************************************************************************/

/* Routine that closes the connection with the XRDev */

int disconnect (void)
{
	int result;
	result = N_DisconnectRobot (1);
	
	switch (result) {
		case N_NO_ERROR:
			printf ("Success!\n");
			break;
		case N_CONNECTION_FAILED:
			printf ("Connection lost!\n");
			break;
		case N_ROBOT_NOT_FOUND:
			printf ("Robot not found!\n");
			break;
	}

	return (result);
}

/****************************************************************************/

void moveaxis (int axis_to_move, int desired_motion, int desired_speed);

/****************************************************************************/

/* Moves the robot in the direction, distance and speed provided */

void move_robot (int direction, int distance, int speed, int teta, int omega) {

	int	ir, sonar, dist;
	struct  Map	mapS, mapI, mapB;

	if (!sensor_on)
		return;		/* returns if the sensors are not on */

	dist = distance;

	MakeSonarMap (&mapS);
	MakeIRMap (&mapI);
	MakeBumperMap (&mapB);
	switch (direction) {
		case RIGHT:
			if ((GetSMapReading (mapS,0,5) > MIN_DIST)
			&& (GetIMapReading (mapI,0,5) < MAX_INT)
			&& (GetBMapReading (mapB,0,5) == N_BUMPER_NONE))	
				moveaxis (X, dist, speed);
			break;
		case LEFT:
			if ((GetSMapReading (mapS,180,5) > MIN_DIST)
			&& (GetIMapReading (mapI,180,5) < MAX_INT)
			&& (GetBMapReading (mapB,0,5) == N_BUMPER_NONE))	
				moveaxis (X, -dist, speed);
			break;
		case FRONT:
			if ((GetSMapReading (mapS,90,5) > MIN_DIST)
			&& (GetIMapReading (mapI,90,5) < MAX_INT)
			&& (GetBMapReading (mapB,0,5) == N_BUMPER_NONE))	
				moveaxis (Y, dist, speed);
			break;
		case BACK:
			if ((GetSMapReading (mapS,270,5) > MIN_DIST)
			&& (GetIMapReading (mapI,270,5) < MAX_INT)
			&& (GetBMapReading (mapB,0,5) == N_BUMPER_NONE))	
				moveaxis (Y, -dist, speed);
			break;
		case SPINP:
			moveaxis (T, teta, omega);
			break;
		case SPINN:
			moveaxis (T, -teta, omega);
			break;
	};

	MakeSonarMap (&mapS);
/*	ShowEvidence (mapS); */

} 

/*****************************************************************************/


/* Moves the specified axis of the robot */

void moveaxis (int axis_to_move, int desired_motion, int desired_speed) {

	struct N_RobotState	*state;
	struct N_Axis		*axis;
	struct N_AxisSet	*axisSet;
	struct Map		mapS, mapI, mapB;
	long			inProgress, i, ir, spin,
				check_sonar, min_dist, initial,
				here, loops;
	char			c;
	int 			angle;
	float			sonar, distance, speed;

	FILE	*outfile;

        outfile = fopen ("/home/tourino/speed.out","w");

	if (!sensor_on)
		return;		/* returns if the sensors are not on */

	min_dist = MIN_DIST;

		/* defines the min_dist as proportional to the speed */

	switch (axis_to_move) {
		case X:
			angle = desired_motion > 0 ? 0 : 180;
			spin = FALSE;
			break;
		case Y:
			angle = desired_motion > 0 ? 90 : 270;
			spin = FALSE;
			break;
		case T:
			spin = TRUE;
			break;
	};

	state = N_GetRobotState(1);
	axisSet = &(state->AxisSet);
	axis = &(axisSet->Axis[axis_to_move]);
	axis->DataActive = TRUE;
	N_GetAxes(1);
	initial = axis->ActualPosition;

	state = N_GetRobotState (1);
	axisSet = &(state->AxisSet);
	axis = &(axisSet->Axis[axis_to_move]);
	axisSet->Global = FALSE;
	axis->DataActive = TRUE;
	axis->Mode=N_AXIS_POSITION_ABSOLUTE;
	axis->DesiredSpeed = 100;		/* start "stopped" */
/*	axis->DesiredSpeed = desired_speed; */
	axis->DesiredPosition = initial+desired_motion;
	axis->Acceleration = 500;
	axis->Update = TRUE;
	N_SetAxes (1);

	loops = 0;

	do {
		loops++;

		/* Reading sensor's data */
		MakeSonarMap (&mapS);
		MakeIRMap (&mapI);

		/* Reading data for the fuzzy controller */
		sonar = GetSMapReading (mapS,angle,5);

		state = N_GetRobotState (1);
		axisSet = &(state->AxisSet);
		axis = &(state->AxisSet.Axis[axis_to_move]);
		axis->DataActive = TRUE;
		N_GetAxes(1);

		here = axis->ActualPosition;
	
		distance = abs(desired_motion - here + initial);
		printf ("distance=%d\n",distance);

		/* call to the fuzzy controller */

		fuzzycontrol (&sonar,&distance,&speed);

		/* setting the new speed from fuzzy controller */

		state = N_GetRobotState (1);
		axisSet = &(state->AxisSet);
		axis = &(state->AxisSet.Axis[axis_to_move]);
		axis->DesiredSpeed = (long) speed;
		axis->DesiredPosition = initial+desired_motion;
		axis->Mode=N_AXIS_POSITION_ABSOLUTE;
		axis->Update = TRUE;
		axis->DataActive = TRUE;
		N_SetAxes (1);

		state = N_GetRobotState (1);
		axis->DataActive = TRUE;
		N_GetAxes (1);
		fprintf (outfile,"%ld %d\n",axis->TimeStamp,axis->ActualVelocity);
		inProgress = axis->InProgress;

		MakeSonarMap (&mapS);
		MakeIRMap (&mapI);
		MakeBumperMap (&mapB);

		check_sonar = spin ? TRUE : ((GetSMapReading
(mapS,angle,9) > min_dist) && (GetIMapReading (mapI,angle,9) < MAX_INT));
/*			&& (GetBMapReading (mapB,0,5) != N_BUMPER_NONE));
*/

/* if spin is true, the routine ignores the sensor's data (the robot will
not collide turning by its axis) */

	}
	while (inProgress && check_sonar && (!stopnow) );
	axis->DesiredSpeed = 0;
	axis->DesiredPosition = here;
	axis->Update = TRUE;
	N_SetAxes (1);

	printf ("loops=%d\n",loops);

	stopnow = FALSE;	/* the robot can move again */

	fflush (outfile);
	fclose (outfile);
}

/****************************************************************************/

/* Gets the lower value of the sonar sets */

void MinSonarSet (struct MySonar sonar1, struct MySonar sonar2, 
	struct MySonar *minsonar) {

int	i;

for (i=0;i<8;i++) {
	minsonar->sonar[i].Reading = sonar1.sonar[i].Reading <
		sonar2.sonar[i].Reading?sonar1.sonar[i].Reading:
		sonar2.sonar[i].Reading;
}

}

/****************************************************************************/

/* Reads sonar data and stores it in the MySonar structure */

void ReadSonarSet (int ss_number, struct MySonar *sonarset) {

struct N_RobotState	*state;	
struct N_Sonar		*sonar;
int			i;

state = N_GetRobotState (1);
sonar = state->SonarController.SonarSet[ss_number].Sonar;
N_GetSonar (1);

for (i=0;i<8;i++)
	sonarset->sonar[i].Reading=sonar[i].Reading;

}

/****************************************************************************/

/* Reads bumper data and stores it in the MyBumper structure */

void ReadBumperSet (int bs_number, struct MyBumper *bumperset) {

struct N_RobotState 	*state;
struct N_Bumper		*bumper;
int			i;

state = N_GetRobotState (1);
bumper = state->BumperController.BumperSet[bs_number].Bumper;
N_GetBumper (1);

for (i=0;i<8;i++)
	bumperset->bumper[i].Reading=bumper[i].Reading;

}

/****************************************************************************/

/* Prints out the sonar data to the stderr output */

void ShowSonarSet (struct MySonar sonarset) {

int	i;

for (i=0;i<8;i++) 
	fprintf (stderr,"%d ",sonarset.sonar[i].Reading);
fprintf (stderr,"\n");

}

/****************************************************************************/

/* Translates the sonar data into MySonar structure to the Map structure */

void MapSonarSet (struct MySonar sonar0, struct MySonar sonar1, 
		struct MySonar sonar2, struct Map *map) {

int	angle, i;

angle = 0;

for (i=6;i<8;i++) {
	map->Reading[angle] = sonar2.sonar[i].Reading;
	angle++;
}

for (i=0;i<8;i++) {
	map->Reading[angle] = sonar0.sonar[i].Reading;
	angle++;
}

for (i=0;i<8;i++) {
	map->Reading[angle] = sonar1.sonar[i].Reading;
	angle++;
}

for (i=0;i<6;i++) {
	map->Reading[angle] = sonar2.sonar[i].Reading;
	angle++;
}

}

/****************************************************************************/

/* Translates the bumper data into MyBumper structure to the Map structure
*/

void MapBumperSet (struct MyBumper bumper0, struct MyBumper bumper1, 
		struct MyBumper bumper2, struct Map *map) {

int	angle, i;

angle = 0;

for (i=6;i<8;i++) {
	map->Reading[angle] = bumper2.bumper[i].Reading;
	angle++;
}

for (i=0;i<8;i++) {
	map->Reading[angle] = bumper0.bumper[i].Reading;
	angle++;
}

for (i=0;i<8;i++) {
	map->Reading[angle] = bumper1.bumper[i].Reading;
	angle++;
}

for (i=0;i<6;i++) {
	map->Reading[angle] = bumper2.bumper[i].Reading;
	angle++;
}

}

/****************************************************************************/

/* Prints out the Map structure to the stderr output */

void ShowMap (struct Map map) {

int 	i;

for (i=0;i<24;i++)
	fprintf (stderr,"%.1f",map.Reading[i]>10000?10:map.Reading[i]/1000.0);

fprintf (stderr,"\n");
}

/****************************************************************************/

/* Returns the greater value from the MyBumper structures */

void MaxBumperSet (struct MyBumper bumper1, struct MyBumper bumper2, 
	struct MyBumper *minbumper) {

int	i;

for (i=0;i<8;i++) {
	minbumper->bumper[i].Reading = bumper1.bumper[i].Reading >
		bumper2.bumper[i].Reading?bumper1.bumper[i].Reading:bumper2.bumper[i].Reading;
}

}

/****************************************************************************/

/* Translate all the bumper sets into the Map structure */

void MakeBumperMap (struct Map *map) {

struct	MyBumper	bs,bi,bumper0,bumper1,bumper2;

ReadBumperSet (0,&bs);
ReadBumperSet (1,&bi);
MaxBumperSet (bs,bi,&bumper0);
ReadBumperSet (2,&bs);
ReadBumperSet (3,&bi);
MaxBumperSet (bs,bi,&bumper1);
ReadBumperSet (4,&bs);
ReadBumperSet (5,&bi);
MaxBumperSet (bs,bi,&bumper2);

MapBumperSet (bumper0,bumper1,bumper2,map);

}

/****************************************************************************/

/* Translate all the sonar sets into the Map structure */

void MakeSonarMap (struct Map *map) {

struct	MySonar		ss,si,sonar0,sonar1,sonar2;

ReadSonarSet (0,&ss);
ReadSonarSet (1,&si);
MinSonarSet (ss,si,&sonar0);
ReadSonarSet (2,&ss);
ReadSonarSet (3,&si);
MinSonarSet (ss,si,&sonar1);
ReadSonarSet (4,&ss);
ReadSonarSet (5,&si);
MinSonarSet (ss,si,&sonar2);

MapSonarSet (sonar0,sonar1,sonar2,map);

}

/****************************************************************************/

/* Displays a bar graph of the sonar data into the stderr output */

void ShowSonarMap (struct Map map) {

short	i,j;
char	s[80];

for (i=0;i<24;i++) {
	strcpy (s,"");
      	for (j=0;j < (map.Reading[i]>10000?10000:map.Reading[i])*0.006;j++)
        	strcat (s,"-");
      	fprintf (stderr,"%d\t%s\n",(map.Reading[i]>10000?10000:map.Reading[i]),s);
}

refresh();
usleep (100000);
clear ();

}

/****************************************************************************/

/* Maps a provided angle to the Map structure's index */

int MapAngle (int angle) {

int a;
	a = ceil (0.06969*angle-1.04545);
	a = a < 0 ? 24+a : a;
	return (a);
}

/****************************************************************************/

/* Gets the value of sonar data in a specified direction */

int GetSMapReading (struct Map map, int angle, int n_sensors) {

int	s, i, a, n, t;

n = n_sensors / 2;
a = MapAngle (angle);
s = 10000;				/* max value to sonar data */
for (i=0;i<2*n+1;i++) {
	t = a-n+i;
	t = t < 0 ? 24-t : t;
	t = t > 23 ?  t-24 : t;
	s = map.Reading[t]<s ? map.Reading[t] : s;
}
return (s);

}

/****************************************************************************/

/* Gets the value of IR data in a specified direction */

int GetIMapReading (struct Map map, int angle, int n_sensors) {

int	s, i, a, n, t;

n = n_sensors / 2;
a = MapAngle (angle);
s = 0;				/* min value for IR data */
for (i=0;i<2*n+1;i++) {
	t = a-n+i;
	t = t < 0 ? 24-t : t;
	t = t > 23 ?  t-24 : t;
	s = map.Reading[t]>s ? map.Reading[t] : s;
}
return (s);

}

/****************************************************************************/

/* Gets the value of Bumper data in a specified direction */

int GetBMapReading (struct Map map, int angle, int n_sensors) {

int	s, i, a, n, t;

n = n_sensors / 2;
a = MapAngle (angle);
s = N_BUMPER_NONE;			/* min value for bumper data */
for (i=0;i<2*n+1;i++) {
	t = a-n+i;
	t = t < 0 ? 24-t : t;
	t = t > 23 ?  t-24 : t;
	s = map.Reading[t]>s ? map.Reading[t] : s;
}
return (s);

}

/****************************************************************************/

/* Displays a gridmap containing the values of Map structure */

void ShowEvidence (struct Map map) {

int 	i,j,l,xm,ym;
int 	EGrid [20][20];
char 	s[80];

for (i=0; i < 20; i++) 
	for (j=0; j < 20; j++) 
		EGrid [i][j] = 0;

EGrid [10][10] = 2;

for (i=0; i < 360; i+= 15) {
	l = GetSMapReading (map, i, 1);
	xm = (3 + (l+300) * cos (i*TO_RADIAN) / 1000) * 3;
	ym = (3 + (l+300) * sin (i*TO_RADIAN) / 1000) * 3;
	if ((xm < 20 && xm >= 0) && (ym >= 0 && ym < 20))
		EGrid [xm][ym] = 1;
}

/* plotting from up to down... */

for (j=19; j >= 0; j--) {
	strcpy (s,"");
	for (i=0; i < 20; i++) {
		if (EGrid [i][j] == 1)
			strcat (s,"++");
		else
			if (EGrid [i][j] == 2)
				strcat (s,"OO");
			else
				strcat (s,"..");
	}
	fprintf (stderr,"%s\n",s);
}

usleep (200000);

}

/****************************************************************************/

/* Returns the greater value from the MyIR structures */

void MaxIRSet (struct MyIR IR1, struct MyIR IR2, 
	struct MyIR *minIR) {

int	i;

for (i=0;i<8;i++) {
	minIR->IR[i].Reading = IR1.IR[i].Reading >
		IR2.IR[i].Reading?IR1.IR[i].Reading:IR2.IR[i].Reading;
}

}

/****************************************************************************/

/* Translates the IR data into the MyIR structure */

void ReadIRSet (int ss_number, struct MyIR *IRset) {

struct N_RobotState	*state;	
struct N_Infrared	*IR;
int			i;

state = N_GetRobotState (1);
IR = state->InfraredController.InfraredSet[ss_number].Infrared;
N_GetInfrared (1);

for (i=0;i<8;i++)
	IRset->IR[i].Reading=IR[i].Reading;

}

/****************************************************************************/

/* Prints out the IR data from a MyIR structure */

void ShowIRSet (struct MyIR IRset) {

int	i;

for (i=0;i<8;i++) 
	fprintf (stderr,"%d ",IRset.IR[i].Reading);
fprintf (stderr,"\n");

}

/****************************************************************************/

/* Translates MyIR structures into the Map structure */

void MapIRSet (struct MyIR IR0, struct MyIR IR1, 
		struct MyIR IR2, struct Map *map) {

int	angle, i;

angle = 0;

for (i=6;i<8;i++) {
	map->Reading[angle] = IR2.IR[i].Reading;
	angle++;
}

for (i=0;i<8;i++) {
	map->Reading[angle] = IR0.IR[i].Reading;
	angle++;
}

for (i=0;i<8;i++) {
	map->Reading[angle] = IR1.IR[i].Reading;
	angle++;
}

for (i=0;i<6;i++) {
	map->Reading[angle] = IR2.IR[i].Reading;
	angle++;
}

}

/****************************************************************************/

/* Prints out the Map structure for the IR data */

void ShowIRMap (struct Map map) {

int 	i;

for (i=0;i<24;i++)
	fprintf (stderr,"%d ",map.Reading[i]);

fprintf (stderr,"\n");
}

/****************************************************************************/

/* Translates all the IR data into the Map structure */

void MakeIRMap (struct Map *map) {

struct	MyIR	ss,si,IR0,IR1,IR2;

ReadIRSet (0,&ss);
ReadIRSet (1,&si);
MaxIRSet (ss,si,&IR0);
ReadIRSet (2,&ss);
ReadIRSet (3,&si);
MaxIRSet (ss,si,&IR1);
ReadIRSet (4,&ss);
ReadIRSet (5,&si);
MaxIRSet (ss,si,&IR2);

MapIRSet (IR0,IR1,IR2,map);

}	

