#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/time.h>
#include <fcntl.h>
#include <stdio.h>
#include <termios.h>
#include <unistd.h>
#include <string.h>
/*
 * writes a char "c" to a file descriptor "fd" and waits for at most
 * "timeout" microseconds for it to be successful.
 * returns -1 if error, 0 if timeout, or 1 if successful
 */

int
wchar (int fd, char *c, int timeout)
{
	int retval;
	fd_set ws;
	struct timeval tout;

	FD_ZERO (&ws);
	FD_SET (fd, &ws);
	tout.tv_sec = timeout / 1000000;
	tout.tv_usec = timeout % 1000000;
	retval = select (fd + 1, NULL, &ws, NULL, &tout);
	if (retval < 0)
		perror ("wchar");
	else if (retval == 0)
		fprintf (stderr, "wchar: timeout\n");
	else {
		retval = write (fd, c, 1);
//		tcdrain (fd);
		if (retval < 0)
			perror ("wchar");
	}
	return retval;
}

/*
 * reads a char "c" from a file descriptor "fd" and waits for at most
 * "timeout" microseconds for it to be successful.
 * returns -1 if error, 0 if timeout, or 1 if successful
 */

int
rchar (int fd, char *c, int timeout)
{
	int retval;
	fd_set rs;
	struct timeval tout;

	FD_ZERO (&rs);
	FD_SET (fd, &rs);
	tout.tv_sec = timeout / 1000000;
	tout.tv_usec = timeout % 1000000;
	retval = select (fd + 1, &rs, NULL, NULL, &tout);
	if (retval < 0)
		perror ("rchar");
	else if (retval == 0)
		fprintf (stderr, "rchar: timeoutt\n");
	else {
		retval = read (fd, c, 1);
		if (retval < 0)
			perror ("rchar");
	}
	return retval;
}

/*
 * writes and then reads a char "c" from a file descriptor "fd" and waits
 * for at most "timeout" microseconds for it to be successful.
 * returns -1 if error, 0 if timeout, or 1 if successful
 */

int
wrchar (int fd, char *c, int timeout)
{
	int retval;
	char rc;

	retval = wchar (fd, c, timeout);
	if (retval > 0) {
		retval = rchar (fd, &rc, timeout);
		if (retval > 0 && rc != *c) {
			*c = rc;
			retval = -1;
		}
	}
	return retval;
}

/*
 * writes a string "s" to a file descriptor "fd" and waits for at most
 * "itimeout" microseconds for the first char to be written and at most
 * "ctimeout" microseconds for each subsequent char to be written.
 * returns -1 if error, 0 if timeout, or 1 if successful
 */

int
wstr (int fd, char *s, int *n, int itimeout, int ctimeout)
{
	int k, retval;

	k = *n;
	retval = *n = 0;
	while (k--) {
		retval = wchar (fd, s, itimeout);
		if (retval <= 0)
			break;
		(*n)++;
		s++;
		itimeout = ctimeout;
	}
	return retval;
}

/*
 * reads a string "s" from a file descriptor "fd" and waits for at most
 * "itimeout" microseconds for the first char to be read and at most
 * "ctimeout" microseconds for each subsequent chat to be read.
 * returns -1 if error, 0 if timeout, or 1 if successful
 */

int
rstr (int fd, char *s, int *n, int itimeout, int ctimeout)
{
	int k, retval;

	k = *n;
	retval = *n = 0;
	while (k--) {
		retval = rchar (fd, s, itimeout);
		if (retval <= 0)
			break;
		(*n)++;
		s++;
		itimeout = ctimeout;
	}
	return retval;
}

/*
 * writes and then reads a string "s" from a file descriptor "fd" and waits
 * for at most "itimeout" microseconds for the first char to be wr and at
 * most "ctimeout" microseconds for each subsequent char to be wr.
 * returns -1 if error, 0 if timeout, or 1 if successful
 */

int
wrstr (int fd, char *s, int *n, int itimeout, int ctimeout)
{
	int k, retval;

	k = *n;
	retval = *n = 0;
	while (k--) {
		retval = wrchar (fd, s, itimeout);
		if (retval <= 0)
			break;
		(*n)++;
		s++;
		itimeout = ctimeout;
	}
	return retval;
}

/*
 * writes a string "s" ending in "f" of at most "n" chars to a file
 * descriptor "fd" and waits for at most "itimeout" microseconds for
 * the first char to be written and at most "ctimeout" microseconds
 * for each subsequent char to be written.
 * returns -1 if error, 0 if timeout, or 1 if successful
 */

int
wzstr (int fd, char *s, int *n, char f, int itimeout, int ctimeout)
{
	int k, retval;

	k = *n;
	retval = *n = 0;
	while (k--) {
		retval = wchar (fd, s, itimeout);
		if (retval <= 0)
			break;
		(*n)++;
		if (*s++ == f)
			break;
		itimeout = ctimeout;
	}
	return retval;
}

/*
 * reads a string "s" ending in "f" of at most "n" chars from a file
 * descriptor "fd" and waits for at most "itimeout" microseconds for
 * the first char to be written and at most "ctimeout" microseconds
 * for each subsequent char to be written.
 * returns -1 if error, 0 if timeout, or 1 if successful
 */

int
rzstr (int fd, char *s, int *n, char f, int itimeout, int ctimeout)
{
	int k, retval;

	k = *n;
	retval = *n = 0;
	while (k--) {
		retval = rchar (fd, s, itimeout);
		if (retval <= 0)
			break;
		(*n)++;
		if (*s++ == f)
			break;
		itimeout = ctimeout;
	}
	return retval;
}

/*
 * writes and then reads a string "s" ending in "f" of at most "n" chars
 * from a file descriptor "fd" and waits for at most "itimeout" microseconds
 * for the first char to be wr and at most "ctimeout" microseconds for each
 * subsequent char to be wr.
 * returns -1 if error, 0 if timeout, or 1 if successful
 */

int
wrzstr (int fd, char *s, int *n, char f, int itimeout, int ctimeout)
{
	int k, retval;

	k = *n;
	retval = *n = 0;
	while (k--) {
		retval = wrchar (fd, s, itimeout);
		if (retval <= 0)
			break;
		(*n)++;
		if (*s++ == f)
			break;
		itimeout = ctimeout;
	}
	return retval;
}

/*
 * simple demo of sending commands to the weboxi...
 * it seems that the protocol is a very simple one, where the machine
 * echoes each char sent until it sees a '!' (which it also echoes).
 * It then interprets it as a command and processes it, sending a string
 * ending in '!' as a result.
 */

#define FCHAR		'!'	/* char used to signal end of command */
#define BUFLEN		127	/* commands and responses max size */

/*
 * returns 1 if success otherwise 0
 */

int
command (int fd, char *cmd, int cmdlen, char *resp, int *resplen, int cmdtimeout, int ctimeout)
{
	int cr, rr, cn, rn, retval;
	char cbuf[BUFLEN + 1], rbuf[BUFLEN + 1];

	cn = cmdlen;
	strncpy (cbuf, cmd, cn);
	cr = wrzstr (fd, cbuf, &cn, FCHAR, ctimeout, ctimeout);
	rn = BUFLEN;
	rr = rzstr (fd, rbuf, &rn, FCHAR, cmdtimeout, ctimeout);
	retval = (cr > 0 && rr > 0) ? 1 : 0;
#ifdef CMDDEBUG
	{
	char buf[BUFLEN + 1];
	strncpy (buf, cmd, cmdlen);
	buf[cmdlen] = '\0';
	/*fprintf (stderr, "Command to send = %s (length = %d)\n", buf,
cmdlen);*/
	strncpy (buf, cbuf, cn);
	buf[cn] = '\0';
	/*fprintf (stderr, "Command    sent = %s (length = %d, result code
= %d)\n", buf, cn, cr);*/
	strncpy (buf, rbuf, rn);
	buf[rn] = '\0';
	/*fprintf (stderr, "Response   read = %s (length = %d, result code
= %d)\n", buf, rn, rr);*/
	
	if (strcmp (buf,"@11s0i!") == 0)
	 fprintf (stderr, "STATUS:Modo Inicial.\n");
	if (strcmp (buf,"@11s1m!") == 0)
	 fprintf (stderr, "Y=0  Status:MODO MANUAL.\n");
	if (strcmp (buf,"@11s2a!") == 0)
	 fprintf (stderr, "STATUS:Modo Manual&Automatico.\n");
	if (strcmp (buf,"@11s3s!") == 0)
	 fprintf (stderr, "STATUS:Modo Execucao Passo-a-Passo.\n");
	if (strcmp (buf,"@11s4c!") == 0)
	 fprintf (stderr, "STATUS:Modo Execucao Continua.\n");
	if (strcmp (buf,"@11s5p!") == 0)
	 fprintf (stderr, "STATUS:Modo Programacao.\n");
	if (strcmp (buf, "@11s6x!") == 0)
	 fprintf (stderr, "STATUS:Comunicacao Externa.\n");
	if (strcmp (buf, "@11s7r!") == 0)
	 fprintf (stderr, "STATUS:Buscando Referencia.\n");
	if (strcmp (buf, "@13e00!") == 0)
         fprintf (stderr, "Erro de Operacao!\n");
	if (strcmp (buf, "@13e05!") == 0)
         fprintf (stderr, "ERRO!!Programa chamado nao existe!\n");
 	if (strcmp (buf, "@13e06!") == 0)
	 fprintf (stderr, "ERRO!!Programa ja existe!\n");
	if (strcmp (buf, "@13e07!") == 0)
         fprintf (stderr, "ERRO!!Memoria Cheia!\n");
	if (strcmp (buf, "@13e09!") == 0)
	 fprintf (stderr, "Erro de Supervisao de
Entrada&Saida!\n");
       	if (strcmp (buf, "@13e10!") == 0)
         fprintf (stderr, "ERRO!!Bateria Fraca!\n");
       	if (strcmp (buf, "@13e15!") == 0)
         fprintf (stderr, "ERRO!!Emergencia Externa!\n");
 	if (strcmp (buf, "@13e50!") == 0)
         fprintf (stderr, "ERRO!!Codigo de desbloqueio invalido!\n");
 	if (strcmp (buf, "@13e54!") == 0)
         fprintf (stderr, "ERRO!!Referencia nao Ativa!\n");  		     
	if (strcmp (buf, "@13e59!") == 0)
	 fprintf (stderr, "ERRO!!Formato errado na recepcao RS-232!\n");
	if (strcmp (buf, "@13e83!") == 0)
	 fprintf (stderr, "ERRO!!Programa selecionado nao existe!\n");
	if (strcmp (buf, "@13e84!") == 0)
	 fprintf (stderr, "ERRO!!Programa nao definido!\n");
	if (strcmp (buf, "@13eD0!") == 0)
	 fprintf (stderr, "ERRO!!Fim de caracteres inesperado!\n");
	if (strcmp (buf, "@13eD1!") == 0)
	 fprintf (stderr, "ERRO!!Fim de sentenca inesperado!\n");
	if (strcmp (buf, "@13eD2!") == 0)
	 fprintf (stderr, "ERRO!!Sentenca invalida!\n");
	if (strcmp (buf, "@13eD3!") == 0)
	 fprintf (stderr, "ERRO!!Muitas teclas na sentenca!\n");
	if (strcmp (buf, "@13eD4!") == 0)
	 fprintf (stderr, "ERRO!!Buffer de comandos lotado!\n");
	if (strcmp (buf, "@13eD5!") == 0)
	 fprintf (stderr, "ERRO!!Overflow do buffer de transmissao!\n");

	}	
#endif
	*resplen = rn;
	strncpy (resp, rbuf, rn);
	return (retval);
}

void
commands (int fd)
{
	char buf[BUFLEN + 1];
	int n;

	tcflush (fd, TCIFLUSH);

	command (fd, "@01T33161683!", 13, buf, &n, 4000000, 400000);
}

#define TTY "/dev/ttyS1"

int
main (int argc, char **agrv)
{
	int fd, cms, cm;
	struct termios ts, t;

	/* open the serial port for rw, nonblocking */
	fd = open (TTY, O_RDWR | O_NONBLOCK);
	if (fd < 0) {
		perror ("open " TTY);
		exit (1);
	}

	/* save the state of the control lines */
	if (ioctl (fd, TIOCMGET, &cms) < 0)
		perror ("TIOCMGET");

	/* save the state of the serial port */
	if (tcgetattr (fd, &ts))
		perror ("tcgetattr");

	/* set the new parameters for the serial port */
	bzero (&t, sizeof (struct termios));
	cfsetispeed (&t, B4800);
	cfsetospeed (&t, B4800);
	t.c_cflag |= CREAD | CS7 | PARENB | CLOCAL;
	if (tcsetattr (fd, TCSANOW, &t))
		perror ("tcsetattr");

	/* raise DTR and RTS */
	cm = cms | TIOCM_DTR | TIOCM_RTS;
	if (ioctl (fd, TIOCMSET, &cm) < 0)
		perror ("TIOCMSET");

	/* execute commands */
	commands (fd);

	/* restore the modem control lines */
	ioctl (fd, TIOCMSET, &cms);

	/* restore the state of the serial port */
	tcsetattr (fd, TCSANOW, &ts);

	/* close the serial port */
	close (fd);

	return (0);
}

